use std::path::PathBuf;

use clap::{
    builder::{
        styling::{AnsiColor, Color, Style},
        Styles,
    },
    Parser,
};

#[cfg(feature = "tdl")]
#[derive(Debug, Parser)]
#[command(styles=get_styles())]
pub struct Cli {
    /// The mode to use
    #[clap(subcommand)]
    pub mode: ModeFile,

    /// The serial port to use
    #[clap(short, long, value_name = "PORT")]
    pub port: Option<String>,

    /// baud rate
    #[clap(short, long, default_value = "115200")]
    pub baud_rate: u32,

    /// disable SYNC MODE (wait for a sync signal from the device before proceding)
    #[clap(short, long)]
    pub no_sync: bool,

    /// The log level
    #[clap(short, long)]
    pub verbose: bool,
}

#[cfg(not(feature = "tdl"))]
#[derive(Debug, Parser)]
#[command(styles=get_styles())]
pub struct Cli {
    /// The serial port to use
    #[clap(short, long, value_name = "PORT")]
    pub port: Option<String>,

    /// baud rate
    #[clap(short, long, default_value = "115200")]
    pub baud_rate: u32,

    /// disable SYNC MODE (wait for a sync signal from the device before proceding)
    #[clap(short, long)]
    pub no_sync: bool,

    /// The CSV file to read from
    //#[clap(value_name = "CSV_FILE")]
    pub csv_file: PathBuf,

    /// The log level
    #[clap(short, long)]
    pub verbose: bool,
}

#[cfg(feature = "tdl")]
#[derive(Debug, Parser)]
pub enum ModeFile {
    /// build mavlink packets from a TDL file
    Tdl {
        /// The TDL file to read from
        tdl_file: PathBuf,

        /// initial GPS latitude
        #[clap(short = 'n', long, value_name = "LAT")]
        lat: f64,

        /// initial GPS longitude
        #[clap(short = 'e', long, value_name = "LON")]
        lon: f64,

        /// initial GPS altitude
        #[clap(short = 'u', long, value_name = "ALT")]
        alt: f64,
    },

    /// parse mavlink packets from a CSV file
    Csv {
        /// The CSV file to read from
        csv_file: PathBuf,
    },
}

pub fn get_styles() -> Styles {
    Styles::styled()
        .usage(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .header(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Yellow))),
        )
        .literal(Style::new().fg_color(Some(Color::Ansi(AnsiColor::Green))))
        .invalid(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .error(
            Style::new()
                .bold()
                .fg_color(Some(Color::Ansi(AnsiColor::Red))),
        )
        .valid(
            Style::new()
                .bold()
                .underline()
                .fg_color(Some(Color::Ansi(AnsiColor::Green))),
        )
        .placeholder(Style::new().fg_color(Some(Color::Ansi(AnsiColor::White))))
}
